import redis

class RedisService:
    def __init__(self, tag:str):
        # self.r = redis.Redis(host='localhost', port=6379, db=0)
        self.tag = tag
        self.tag_hash = f"{tag}_hash"

    def initRedis(self, host:str, port:int):
        self.r = redis.Redis(host=host, port=port, db=0)

    def getRankKey(self, rankType:str):
        rankKey = f"{self.tag}:{rankType}"
        return rankKey

    def getRankHashKey(self, rankType:str):
        rankHashKey = f"{self.tag_hash}:{rankType}"
        return rankHashKey

    def updateRankScore(self, rankType:str, playerId:str, score:int, playerInfo:str = None):
        '''更新玩家排行榜分数'''
        # 使用pipeline，将多条命令打包执行
        self.r.pipeline().multi()

        # 更新玩家排行榜分数
        rankKey = self.getRankKey(rankType)
        self.r.zadd(rankKey, {playerId: score})

        # 更新玩家简要信息
        if playerInfo is not None:
            hashName = self.getRankHashKey(rankType)
            self.r.hset(hashName, playerId, playerInfo)
        self.r.pipeline().execute()

    def getRankList(self, rankType:str, start:int, end:int):
        '''获取排行榜列表'''
        rankKey = self.getRankKey(rankType)
        rankHashKey = self.getRankHashKey(rankType)
        self.r.pipeline().multi()
        tops = self.r.zrevrange(rankKey, start, end, withscores=True)
        result = []
        for rank, (user0, score) in enumerate(tops, start=1):
            user = user0.decode('utf-8')
            playerInfo0 = self.r.hget(rankHashKey, user)
            playerInfo = playerInfo0.decode("utf-8")
            data = {
                "rank": rank,
                "player_id": user,
                "score": int(score),
                "player_info": playerInfo,
            }
            result.append(data)
        self.r.pipeline().execute()
        return result

    def getPlayerRankInfo(self, rankType:str, playerId:str):
        '''查询玩家排行榜信息'''
        rankKey = self.getRankKey(rankType)
        rankHashKey = self.getRankHashKey(rankType)
        # 查看玩家排名
        rankInfo = self.r.zrevrank(rankKey, playerId, True)
        if rankInfo is None:
            return {}

        (rank, score0) = self.r.zrevrank(rankKey, playerId, True)
        score = score0.decode('utf-8')
        rank += 1
        if rank is not None and score is not None:
            print(f"Player {playerId} is ranked #{rank} with score {int(score)}")
        else:
            print(f"Player {playerId} not found in the leaderboard")

        playerInfoStr = self.r.hget(rankHashKey, playerId)
        data = {
            "player_id": playerId,
            "rank": rank,
            "player_info": playerInfoStr.decode('utf-8'),

        }
        return data

    def deleteRankScore(self, rankType:str, playerId:str):
        rankKey = self.getRankKey(rankType)
        rankHashKey = self.getRankHashKey(rankType)

        self.r.pipeline().multi()
        self.r.zrem(rankKey, playerId)
        self.r.hdel(rankHashKey, playerId)
        self.r.pipeline().execute()

    def clearByRankType(self, rankType:int):
        '''根据排行榜类型清空排行榜'''
        rankKey = self.getRankKey(rankType)
        rankHashKey = self.getRankHashKey(rankType)
        self.r.delete(rankKey, rankHashKey)